#ifndef _OPENDRIVE_LANE_H
#define _OPENDRIVE_LANE_H

#ifdef _HAVE_XSD
#include "../src/xml/OpenDRIVE_1.4H.hxx"
#endif

#include "opendrive_road_node.h"
#include "opendrive_road_segment.h"

class COpendriveRoadSegment;

class COpendriveLane
{
  friend class COpendriveRoadSegment;
  private:
    std::vector<COpendriveRoadNode *> nodes;
    COpendriveLane *left_lane;
    opendrive_mark_t left_mark;
    COpendriveLane *right_lane;
    opendrive_mark_t right_mark;
    COpendriveRoadSegment *segment;
    double scale_factor;
    double resolution;
    double width;
    double speed;
    double offset;
    int id;
  protected:
    COpendriveLane();
    COpendriveLane(const COpendriveLane &object,std::map<COpendriveRoadNode *,COpendriveRoadNode *> &node_refs,COpendriveRoadSegment *segment_ref);
    void load(const right::lane_type &lane_info,COpendriveRoadSegment *segment);
    void link_neightbour_lane(COpendriveLane *lane);
    void link_lane(COpendriveLane *lane,opendrive_mark_t mark);
    void add_node(COpendriveRoadNode *node);
    void set_resolution(double res);
    void set_scale_factor(double scale);
    void set_offset(double offset);
    void update_references(std::map<COpendriveLane *,COpendriveLane *> &lane_refs);
  public:
    unsigned int get_num_nodes(void) const;
    const COpendriveRoadNode &get_node(unsigned int index) const;
    const COpendriveRoadSegment &get_segment(void) const;
    double get_width(void) const;
    double get_speed(void) const;
    double get_offset(void) const;
    int get_id(void) const;
    TOpendriveWorldPoint get_start_point(void);
    TOpendriveWorldPoint get_end_point(void);
    double get_length(void);
    friend std::ostream& operator<<(std::ostream& out, COpendriveLane &lane);
    ~COpendriveLane();
};

#endif
