#ifndef _OPENDRIVE_LINK_H
#define _OPENDRIVE_LINK_H

#include "g2_spline.h"
#include "opendrive_line.h"
#include "opendrive_spiral.h"
#include "opendrive_arc.h"
#include "opendrive_param_poly3.h"

class COpendriveRoadNode;

class COpendriveLink
{
  friend class COpendriveRoadNode;
  private:
    COpendriveRoadNode *prev;
    COpendriveRoadNode *next;
    CG2Spline *spline;
    opendrive_mark_t mark;
    double resolution;
    double scale_factor;    
  protected:
    COpendriveLink();
    COpendriveLink(const COpendriveLink &object);
    void set_prev(COpendriveRoadNode *node);
    void set_next(COpendriveRoadNode *node);
    void set_road_mark(opendrive_mark_t mark);
    void set_resolution(double res);
    void set_scale_factor(double scale);
    void generate(void);
    void update_references(std::map<COpendriveRoadNode *,COpendriveRoadNode *> refs);
  public:
    const COpendriveRoadNode &get_prev(void) const;
    const COpendriveRoadNode &get_next(void) const;
    opendrive_mark_t get_road_mark(void) const;
    double get_resolution(void) const;
    double get_scale_factor(void) const;
    double find_closest_world_point(TOpendriveWorldPoint &world,TPoint &point);
    double find_closest_local_point(TOpendriveLocalPoint &local,TPoint &point);
    double get_track_point_world(TOpendriveTrackPoint &track,TOpendriveWorldPoint &world);
    double get_track_point_local(TOpendriveTrackPoint &track,TOpendriveLocalPoint &local);
    double get_length(void) const;
    friend std::ostream& operator<<(std::ostream& out, COpendriveLink &link);
    ~COpendriveLink();
};

#endif
