#ifndef _OPENDRIVE_ROAD_SEGMENT_H
#define _OPENDRIVE_ROAD_SEGMENT_H

#ifdef _HAVE_XSD
#include "../src/xml/OpenDRIVE_1.4H.hxx"
#endif

#include "opendrive_lane.h"
#include "opendrive_signal.h"
#include "opendrive_object.h"
#include "opendrive_road.h"

class COpendriveLane;
class COpendriveRoad;

class COpendriveRoadSegment
{
  friend class COpendriveRoad;
  private:
    std::map<int,COpendriveLane *> lanes;
    COpendriveRoad *parent_road;
    double resolution;
    double scale_factor;
    double min_road_length;
    int num_right_lanes;
    int num_left_lanes;
    std::vector<COpendriveSignal *> signals;
    std::vector<COpendriveObject *> objects;
    std::vector<COpendriveRoadSegment *> connecting;
    std::string name;
    unsigned int id;
  protected:
    COpendriveRoadSegment();
    COpendriveRoadSegment(const COpendriveRoadSegment &object,std::map<COpendriveRoadNode *,COpendriveRoadNode *> &node_refs,COpendriveRoad *road_ref);
    void load(OpenDRIVE::road_type &road_info);
    void free(void);
    void set_resolution(double res);
    void set_scale_factor(double scale);
    void set_min_road_length(double length);
    void set_parent_road(COpendriveRoad *parent);
    void update_references(std::map<COpendriveRoadSegment *,COpendriveRoadSegment *> &segment_refs);
    void add_lanes(lanes::laneSection_type &lane_section);
    void add_nodes(OpenDRIVE::road_type &road_info);
    void link_neightbour_lanes(lanes::laneSection_type &lane_section);
    void link_segment(COpendriveRoadSegment &segment);
    void link_segment(COpendriveRoadSegment &segment,int from, int to);
  public:
    std::string get_name(void) const;
    unsigned int get_id(void) const;
    unsigned int get_num_right_lanes(void) const;
    unsigned int get_num_left_lanes(void) const;
    const COpendriveLane &get_lane(int index) const;
    const COpendriveRoad &get_parent_road(void) const;
    unsigned int get_num_signals(void) const;
    const COpendriveSignal &get_signal(unsigned int index) const;
    unsigned int get_num_obstacles(void) const;
    const COpendriveObject &get_object(unsigned int index) const;
    unsigned int get_num_connecting(void) const;
    const COpendriveRoadSegment &get_connecting(unsigned int index) const;
    friend std::ostream& operator<<(std::ostream& out, COpendriveRoadSegment &segment);
    ~COpendriveRoadSegment();
};

#endif
