#include "dijkstra.h"
#include <limits>

CDijkstra::CDijkstra()
{

}

void CDijkstra::sort_rows(Eigen::MatrixXd &input)
{
  int i;
  unsigned int j;
  double min,index;

  for(j=1;j<input.rows();j++)
  {
    min=input(j,0);
    index=input(j,1);
    i=j-1;
    while(i>=0 && input(i,0)>min)
    {
      input(i+1,0)=input(i,0);
      input(i+1,1)=input(i,1);
      i--;
    }
    input(i+1,0)=min;
    input(i+1,1)=index;
  } 
}

void CDijkstra::get_path(std::vector<unsigned int> &path,Eigen::MatrixXd &w,unsigned int start_node, unsigned int end_node)
{
  int index;

  if(w(1,end_node)==w(w.rows()-1,end_node))
    path.push_back(start_node);
  else
  {
    index=w.rows()-1;
    while(index>=0)
    {
      if(w(index,end_node)<w(index-1,end_node))
      {
        path.push_back(w(index,0));
        this->get_path(path,w,start_node,w(index,0));
        index=-1;
      }
      else
        index--;
    }
  }
}

double CDijkstra::find_shortest_path(Eigen::MatrixXd &graph,unsigned int start_node,unsigned int end_node,std::vector<unsigned int> &path)
{
  Eigen::MatrixXd w=Eigen::MatrixXd::Zero(graph.rows(),graph.cols());
  Eigen::MatrixXd d=Eigen::MatrixXd(graph.rows(),2);
  Eigen::MatrixXd d2,new_d2;
  unsigned int i,j,k,l;

  path.clear();
  if(start_node==end_node)
  {
    path.push_back(end_node);
    return 0.0;
  }
  // set Inf cost to all cells with 0
  for(i=0;i<graph.rows();i++)
    for(j=0;j<graph.cols();j++)
      if(graph(i,j)==0.0)
        graph(i,j)=std::numeric_limits<double>::max()/2.0;
  for(i=1;i<graph.rows();i++)
  {
    w(0,i)=i;
    w(1,i)=graph(0,i);
  }
  for(i=0;i<graph.rows();i++)
  {
    d(i,0)=graph(0,i);
    d(i,1)=i;
  }
  d2=d.block(1,0,d.rows()-1,2);
  l=1;
  while(l<(graph.rows()-1))
  {
    l++;
    this->sort_rows(d2);
    k=d2(0,1);
    w(l,0)=k;
    new_d2=d2.block(1,0,d2.rows()-1,2);
    for(i=0;i<new_d2.rows();i++)
    {
      if(d(new_d2(i,1),0)>(d(k,0)+graph(k,new_d2(i,1))))
      {
        d(new_d2(i,1),0)=d(k,0)+graph(k,new_d2(i,1));
        new_d2(i,0)=d(new_d2(i,1),0); 
      }
    }
    for(i=1;i<graph.rows();i++)
      w(l,i)=d(i,0);
    d2=new_d2;
  }
  path.clear();
  // find the last reachable cell (preferably on the same target lane)
  while(w(1,end_node)==w(w.rows()-1,end_node) && w(1,end_node)==std::numeric_limits<double>::max()/2.0)
    end_node--;
  path.push_back(end_node); 
  this->get_path(path,w,start_node,end_node);

  return w(w.rows()-1,end_node);
}

CDijkstra::~CDijkstra()
{

}

