#include "opendrive_arc.h"
#include <cmath>

COpendriveArc::COpendriveArc()
{
  this->curvature=0.0;
}

COpendriveArc::COpendriveArc(double min_s, double max_s, double x, double y, double heading, double curvature) : COpendriveGeometry(min_s,max_s,x,y,heading)
{
  this->curvature=curvature;
}

COpendriveArc::COpendriveArc(const COpendriveArc &object) : COpendriveGeometry(object)
{
  this->curvature=object.curvature;
}

bool COpendriveArc::transform_local_pose(TOpendriveTrackPoint &track,TOpendriveLocalPoint &local) const
{
  double alpha;
  bool pos_arc;

  alpha = std::fabs((track.s-this->min_s/this->scale_factor)*this->curvature*this->scale_factor);
  pos_arc = (this->curvature < 0.0 ? false : true);
  local.u = std::sin(alpha)/(this->curvature*this->scale_factor) - track.t*std::sin(alpha)*(pos_arc ? 1 : -1);
  local.v = (1 - std::cos(alpha))*(pos_arc ? 1 : -1)/(this->curvature*this->scale_factor) + track.t*std::cos(alpha);
  local.heading = normalize_angle(track.heading + alpha*(pos_arc ? 1 : -1));

  return true;
}

void COpendriveArc::print(std::ostream &out)
{
  COpendriveGeometry::print(out);
  out << "          curvature = " << this->get_curvature() << std::endl;
}

std::string COpendriveArc::get_name(void)
{
  return std::string("arc");
}

void COpendriveArc::load_params(const planView::geometry_type &geometry_info)
{
  this->curvature = (geometry_info.arc().get().curvature().present() ? geometry_info.arc().get().curvature().get() : 0.0);
}

COpendriveGeometry *COpendriveArc::clone(void)
{
  COpendriveArc *new_arc=new COpendriveArc(*this);

  return new_arc;
}

double COpendriveArc::get_curvature(void)
{
  return this->curvature*this->scale_factor;
}

void COpendriveArc::operator=(const COpendriveArc &object)
{
  COpendriveGeometry::operator=(object);
  this->curvature=object.curvature;
}

COpendriveArc::~COpendriveArc()
{
  this->curvature=0.0;
}

